import 'dart:convert';
import 'package:http/http.dart';

import 'package:anilibria/src/models/models.dart';
import 'package:anilibria/src/enums/enums.dart';
import 'package:anilibria/src/utils/utils.dart';

/// Anilibria API wrapper.
class Anilibria {
  final Uri _baseUrl;
  final Client _client;

  Anilibria(this._baseUrl) : _client = Client();

  Future<Title> getTitle({
    int? id,
    String? code,
    int? torrentId,
    List<String>? filter,
    List<String>? remove,
    List<Include>? include,
    DescriptionType? descriptionType,
    PlaylistType? playlistType,
  }) async {
    final res = await _client.get(getUrl(_baseUrl, '/title', {
      if (id != null) 'id': id.toString(),
      if (code != null) 'code': code,
      if (filter != null) 'filter': filter.join(','),
      if (remove != null) 'remove': remove.join(','),
      if (include != null) 'include': include.join(','),
      if (descriptionType != null) 'description_type': descriptionType.value,
      if (playlistType != null) 'playlist_type': playlistType.value,
    }));
    final json = jsonDecode(utf8.decode(res.bodyBytes));

    return Title.fromJson(json);
  }

  Future<List<Title>> getTitles({
    List<int>? idList,
    List<String>? codeList,
    List<String>? filter,
    List<String>? remove,
    List<Include>? include,
    DescriptionType? descriptionType,
    PlaylistType? playlistType,
  }) async {
    final res = await _client.get(getUrl(_baseUrl, '/title/list', {
      if (idList != null) 'id_list': idList.join(','),
      if (codeList != null) 'code_list': codeList.join(','),
      if (filter != null) 'filter': filter.join(','),
      if (remove != null) 'remove': remove.join(','),
      if (include != null) 'include': include.join(','),
      if (descriptionType != null) 'description_type': descriptionType.value,
      if (playlistType != null) 'playlist_type': playlistType.value,
    }));
    final json = jsonDecode(utf8.decode(res.bodyBytes));

    return [for (final e in json) Title.fromJson(e)];
  }

  Future<Page<Title>> getUpdates({
    List<String>? filter,
    List<String>? remove,
    List<Include>? include,
    int? limit,
    DateTime? since,
    DescriptionType? descriptionType,
    PlaylistType? playlistType,
    int? after,
    int? page,
    int? itemsPerPage,
  }) async {
    final res = await _client.get(getUrl(_baseUrl, '/title/updates', {
      if (filter != null) 'filter': filter.join(','),
      if (remove != null) 'remove': remove.join(','),
      if (include != null) 'include': include.join(','),
      if (limit != null) 'limit': limit.toString(),
      if (since != null) 'since': since.millisecondsSinceEpoch.toString(),
      if (descriptionType != null) 'description_type': descriptionType.value,
      if (playlistType != null) 'playlist_type': playlistType.value,
      if (after != null) 'after': after.toString(),
      if (page != null) 'page': page.toString(),
      if (itemsPerPage != null) 'items_per_page': itemsPerPage.toString(),
    }));
    final json = jsonDecode(utf8.decode(res.bodyBytes));

    return Page.fromJson(json, Title.fromJson);
  }

  Future<List<Title>> getChanges({
    List<String>? filter,
    List<String>? remove,
    List<Include>? include,
    int? limit,
    DateTime? since,
    DescriptionType? descriptionType,
    int? after,
    int? page,
    int? itemsPerPage,
  }) async {
    final res = await _client.get(getUrl(_baseUrl, '/title/changes', {
      if (filter != null) 'filter': filter.join(','),
      if (remove != null) 'remove': remove.join(','),
      if (include != null) 'include': include.join(','),
      if (limit != null) 'limit': limit.toString(),
      if (since != null) 'since': since.millisecondsSinceEpoch.toString(),
      if (descriptionType != null) 'description_type': descriptionType.value,
      if (after != null) 'after': after.toString(),
      if (page != null) 'page': page.toString(),
      if (itemsPerPage != null) 'items_per_page': itemsPerPage.toString(),
    }));
    final json = jsonDecode(utf8.decode(res.bodyBytes));

    return [for (final e in json) Title.fromJson(e)];
  }

  Future<Page<Title>> searchTitles({
    String? search,
    List<String>? years,
    List<SeasonCode>? seasonCode,
    List<String>? genres,
    List<String>? voice,
    List<String>? translator,
    List<String>? editing,
    List<String>? decor,
    List<String>? timing,
    List<String>? filter,
    List<String>? remove,
    List<Include>? include,
    DescriptionType? descriptionType,
    PlaylistType? playlistType,
    int? limit,
    int? after,
    int? page,
    int? itemsPerPage,
  }) async {
    final res = await _client.get(getUrl(_baseUrl, '/title/search', {
      if (search != null) 'search': search,
      if (seasonCode != null)
        'season_code': seasonCode.map((e) => e.value.toString()).join(', '),
      if (genres != null) 'genres': genres.join(', '),
      if (voice != null) 'voice': voice.join(', '),
      if (translator != null) 'translator': translator.join(', '),
      if (editing != null) 'editing': editing.join(', '),
      if (decor != null) 'decor': decor.join(', '),
      if (timing != null) 'timing': timing.join(', '),
      if (filter != null) 'filter': filter.join(','),
      if (remove != null) 'remove': remove.join(','),
      if (include != null) 'include': include.join(','),
      if (descriptionType != null) 'description_type': descriptionType.value,
      if (playlistType != null) 'playlist_type': playlistType.value,
      if (limit != null) 'limit': limit.toString(),
      if (after != null) 'after': after.toString(),
      if (page != null) 'page': page.toString(),
      if (itemsPerPage != null) 'items_per_page': itemsPerPage.toString(),
    }));
    final json = jsonDecode(utf8.decode(res.bodyBytes));

    return Page.fromJson(json, Title.fromJson);
  }

  Future<Title> getRandomTitle({
    List<String>? filter,
    List<String>? remove,
    List<Include>? include,
    DescriptionType? descriptionType,
    PlaylistType? playlistType,
  }) async {
    final res = await _client.get(getUrl(_baseUrl, '/title/random', {
      if (filter != null) 'filter': filter.join(','),
      if (remove != null) 'remove': remove.join(','),
      if (include != null) 'include': include.join(','),
      if (descriptionType != null) 'description_type': descriptionType.value,
      if (playlistType != null) 'playlist_type': playlistType.value,
    }));
    final json = jsonDecode(utf8.decode(res.bodyBytes));

    return Title.fromJson(json);
  }

  Future<Page<Youtube>> getYoutube({
    List<String>? filter,
    List<String>? remove,
    int? limit,
    DateTime? since,
    int? after,
    int? page,
    int? itemsPerPage,
  }) async {
    final res = await _client.get(getUrl(_baseUrl, '/youtube', {
      if (filter != null) 'filter': filter.join(','),
      if (remove != null) 'remove': remove.join(','),
      if (limit != null) 'limit': limit.toString(),
      if (since != null) 'since': since.millisecondsSinceEpoch.toString(),
      if (after != null) 'after': after.toString(),
      if (page != null) 'page': page.toString(),
      if (itemsPerPage != null) 'items_per_page': itemsPerPage.toString(),
    }));
    final json = jsonDecode(utf8.decode(res.bodyBytes));

    return Page.fromJson(json, Youtube.fromJson);
  }

  Future<List<Schedule>> getSchedule({
    List<String>? days,
    List<String>? filter,
    List<String>? remove,
    List<Include>? include,
    DescriptionType? descriptionType,
    PlaylistType? playlistType,
  }) async {
    final res = await _client.get(getUrl(_baseUrl, '/title/schedule', {
      if (days != null) 'days': days.join(','),
      if (filter != null) 'filter': filter.join(','),
      if (remove != null) 'remove': remove.join(','),
      if (include != null) 'include': include.join(','),
      if (descriptionType != null) 'description_type': descriptionType.value,
      if (playlistType != null) 'playlist_type': playlistType.value,
    }));
    final json = jsonDecode(utf8.decode(res.bodyBytes));

    return [for (final e in json) Schedule.fromJson(e)];
  }

  void dispose() {
    _client.close();
  }
}
